@@ -3,42 +3,54 @@ package Mojolicious::Plugin::BModel;
 use 5.010;
 use strict;
 use warnings;
-use Carp qw/ croak /;
+use Carp qw/ carp croak /;
 use File::Find qw/ find /;
+use List::Util qw/ first /;
+use File::Spec;
 
 use Mojo::Loader;
 use Mojo::Base 'Mojolicious::Plugin';
 
-our $VERSION = '0.09';
+our $VERSION = '0.1';
 
-my $CREATE_DIR = 1;
-my $MODEL_DIR  = 'Model'; # directory of poject for the Model-modules
+my $DEFAULT_CREATE_DIR = 1;
+my $DEFAULT_MODEL_DIR  = 'Model'; # directory of poject for the Model-modules
 my %MODULES    = ();
 my $BASE_MODEL = 'Mojolicious::BModel::Base';
 
 sub register {
     my ( $self, $app, $conf ) = @_;
 
-    my $app_name      = ref $app; # name of calling app
-    my $path_to_model = $app->home->lib_dir . '/' . $app_name . '/' . $MODEL_DIR;
-    my $dir_exists    = $self->check_model_dir( $path_to_model );
-    my $create_dir    = $conf->{create_dir} || $CREATE_DIR;
+    my $app_name  = ref $app; # name of calling Mojo app
 
-    if ( ! $dir_exists && ! $create_dir ) {
-        warn "Directory $app_name/$MODEL_DIR does not exist";
+    # namespace path to Models dir. By default it is the name of application, but it can be redefine.
+    # For example, namespace can be MyPath::ToModel or any other
+    my $namespace = $conf->{namespace} ? $conf->{namespace} : $app_name;
+    if ( ! $self->_check_namespace( $namespace ) ) {
+        croak "Wrong format of namespace $namespace. Exit";
+    }
+
+    my $namespace_path = $self->_convert_namespace_to_path( $namespace );
+
+    # work with considering operation system
+    my $path_to_model = File::Spec->catfile( $app->home->child('lib')->to_string, $namespace_path, $DEFAULT_MODEL_DIR );
+    my $dir_exists    = $self->_check_model_dir( $path_to_model );
+    my $to_create_dir = exists $conf->{create_dir} ? $conf->{create_dir} : $DEFAULT_CREATE_DIR;
+
+    if ( ! ( $dir_exists && $to_create_dir ) ) {
+        carp "Directory " . File::Spec->catfile( $namespace_path, $DEFAULT_MODEL_DIR ) . " does not exist";
         return 1;
     }
-    elsif ( ! $dir_exists && $create_dir ) {
+    elsif ( ! $dir_exists && $to_create_dir ) {
         mkdir $path_to_model or croak "Could not create directory $path_to_model : $!";
     }
 
-    $self->load_models( $path_to_model, $app_name, $app );
+    $self->load_models( $path_to_model, $namespace, $app );
 
     $app->helper(
         model => sub {
             my ( $self, $model_name ) = @_;
-            $model_name =~ s/\/+/::/g;
-            croak "Unknown model $model_name" unless $MODULES{ $model_name };
+            croak "Unknown model $model_name" if ! exists $MODULES{ $model_name };
             return $MODULES{ $model_name };
         }
     );
@@ -46,13 +58,58 @@ sub register {
     return 1;
 }
 
-sub check_model_dir {
+# check a path to the 'Model' dir
+sub _check_model_dir {
     my ( $self, $path_to_model ) = @_;
 
     return 1 if -e $path_to_model && -d $path_to_model;
     return;
 }
 
+# check for validation of namespace
+sub _check_namespace {
+    my ( $self, $namespace ) = @_;
+
+    my @splitted_namespace = $self->_separate_namespace_name( $namespace );
+    return if ! scalar @splitted_namespace;
+
+    my $found_wrong_name = first { $_ !~ m/[a-zA-Z0-9]/ } @splitted_namespace;
+    return if $found_wrong_name;
+
+    return 1;
+}
+
+# convert name like 'Aaaa::Bbbbb::Cccc' to 'Aaaa/Bbbbb/Cccc' for Unix-like OS or 'Aaaa\Bbbbb\Cccc' for Windows
+sub _convert_namespace_to_path {
+    my ( $self, $namespace ) = @_;
+
+    my @splitted_namespace = $self->_separate_namespace_name( $namespace );
+    my $path = scalar @splitted_namespace == 1 ? $splitted_namespace[0] : File::Spec->catfile( @splitted_namespace );
+
+    return $path;
+}
+
+sub _separate_namespace_name {
+    my ( $self, $namespace ) = @_;
+
+    my @splitted_name = split( /\:\:/, $namespace );
+
+    return @splitted_name;
+}
+
+sub _convert_model_dirs_array {
+    my ( $self, @dirs ) = @_;
+
+    my $canonical_name;
+
+    for my $dir ( @dirs ) {
+        next if $dir =~ m/^\s*$/;
+        $canonical_name = ( defined $canonical_name ) ? $canonical_name . '::' . $dir : $dir;
+    }
+
+    return $canonical_name;
+}
+
 sub find_models {
     my ( $self, $path_to_model, $model_path ) = @_;
 
@@ -63,9 +120,16 @@ sub find_models {
         sub {
             return if ! -d $File::Find::name || $File::Find::name eq $path_to_model;
             my $dir_name = $File::Find::name;
-            $dir_name =~ s/$path_to_model\/?(.+)/$1/;
-            $dir_name =~ s/(\/)+/::/g;
-            push @model_dirs, $model_path . '::' . $dir_name;
+            $dir_name =~ s/$path_to_model//; # remove path to model from directory
+
+            # split dir path name and convert it into name like 'aaa::bbb::ccc'
+            my @dirs = File::Spec->splitdir( $dir_name );
+            my $canonical_dir_name = $self->_convert_model_dirs_array( @dirs );
+            if ( ! $canonical_dir_name ) {
+                carp "Cannot parse dir name $dir_name";
+                return;
+            }
+            push @model_dirs, $model_path . '::' . $canonical_dir_name;
         },
         ( $path_to_model )
     );
@@ -75,9 +139,9 @@ sub find_models {
 
 # recursive search and download modules with models
 sub load_models {
-    my ( $self, $path_to_model, $app_name, $app ) = @_;
+    my ( $self, $path_to_model, $namespace, $app ) = @_;
 
-    my $model_path = "$app_name\::$MODEL_DIR";
+    my $model_path = $namespace . '::' . $DEFAULT_MODEL_DIR;
     my @model_dirs = @{ $self->find_models( $path_to_model, $model_path ) };
 
     my $base_load_err = Mojo::Loader::load_class( $BASE_MODEL );
@@ -119,7 +183,7 @@ Mojolicious::Plugin::BModel - Catalyst-like models in Mojolicious
     sub startup {
         my $self = shift;
 
-        $self->plugin( 'BModel', { create_dir => 1 } );
+        $self->plugin( 'BModel', { %Options } );
     }
 
     # in controller:
@@ -129,7 +193,7 @@ Mojolicious::Plugin::BModel - Catalyst-like models in Mojolicious
         my $config_data = $self->model('MyModel')->get_conf_data('field');
     }
 
-    # in <your_app>/lib/Model/MyModel.pm:
+    # in <your_app>/lib/<namespace>/Model/MyModel.pm:
 
     use Mojo::Base 'Mojolicious::BModel::Base';
 
@@ -146,15 +210,27 @@ Mojolicious::Plugin::BModel - Catalyst-like models in Mojolicious
     and use this one by the method 'model' of a controller object.
     This approach is using in the L<Catalyst framework|https://metacpan.org/pod/Catalyst>.
 
+    All model classes have to be inherited from the class 'Mojolicious::BModel::Base', examples see below.
+
+    This module works in Unix-like and Windows systems.
+
 =head2 Options
 
 =over
 
 =item B<create_dir>
 
-    A flag that determines automatically create the folder '<yourapp>/lib/Model'
+    A boolean flag that determines automatically create the folder '<yourapp>/lib/<namespace>/Model'
     if it does not exist. 0 - do not create, 1 - create. Enabled by default
 
+=item B<namespace>
+
+    A place in the '/lib' folder of application where there are your model classes. By default it is the name of application.
+    The value of this parameter should be in the format with a delimiter '::', for example 'Aaaaa::Bbbb::Cccc'.
+    This string(in format 'Aaaaa::Bbbb::Cccc') will be converted to the path 'Aaaaa/Bbbb/Cccc'(or 'Aaaaa\Bbbb\Cccc'
+    for Microsoft Windows) and the absolute path to the your Model dir will looks like '<your app>/lib/Aaaaa/Bbbb/Cccc/Model'
+    and all Model classes will be sought in this direcory.
+
 =back
 
 =cut
@@ -209,12 +285,11 @@ Mojolicious::Plugin::BModel - Catalyst-like models in Mojolicious
 
     # end of edit file
 
-    # When you connect, the plugin will check if the folder "lib/Model".
-    # If the folder does not exist, create it.
-    # If the 'use_base_model' is set to true will be loaded
-    # module "Mojolicious::BModel::Base" with the base model.
-    # Method 'app' base model will contain a link to your application.
-    # Method 'config' base model will contain a link to config of yor application.
+    # When you connect, the plugin will check the folder "lib/MyApp/Model" exists.
+    # By defaut the namespace 'MyApp' will be used.
+    # If the folder 'lib/MyApp/Model' does not exist, this will be created.
+    # The method 'app' of base model contains a link to your application.
+    # The method 'config' of base model contains a link to config of yor application.
 
     # create a new model
     % touch lib/MyApp/Model/MyModel.pm
@@ -247,7 +322,7 @@ Mojolicious::Plugin::BModel - Catalyst-like models in Mojolicious
 
 =head1 LICENSE
 
-Copyright (C) 2016 Alexander Ruzhnikov.
+Copyright (C) 2015-2017 Alexander Ruzhnikov.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
